#' Conditional Distribution based on asymptotic distribution
#' @description Estimate conditional distribution of optimal
#' tuning paramter based on asymptotic distribution
#' @importFrom glmpath glmpath
#' @importFrom nloptr auglag
#' @param X    input matrix or design matrix
#' @param y    response variable
#' @param wn   weight function, default is \eqn{log(n) / n}, which correponds to
#' BIC criterion. For AIC, choose \eqn{2 / n}.
#' @param alpha value to specify confidence level.  it
#' corrsponds confidence interval \eqn{ (1 - alpha) * 100\% }
#' @param ... Other arguments that can be passed to \code{glmpath}
#' @return
#' \item{prob}{Estimated conditional probablity of tuning parameter being
#' selected as the optimal tuning paramter}
#' \item{CI}{Confidence intervals for conditional probality based on normal
#' assumption}
#' \item{model}{a \code{glmpath} object, which is the same the fit
#' \code{glmpath(X, y, family = 'gaussian')}}
#' \item{alpha}{the specified alpha level}
#' \item{wn}{the specified wn}
#' @export
#' @examples
#' set.seed(0622)
#' X <- matrix(rnorm(50*10), 50, 10)
#' y <- X[,1:3] %*% c(1, 0.5, 1) + rnorm(50)
#' asym_ci(X,y)

asym_ci<- function(X, y, wn, alpha = 0.1, ...){

  # Check dimensions #
  if(is.null(dim(X))) stop("X should be a matrix");
  dimy=dim(y)
  nrowy=ifelse(is.null(dimy),length(y),dimy[1])
  if(nrowy!=dim(X)[1])
    stop(paste('number of observations does not match for X and y'))

  p <- ncol(X)
  n <- nrow(X)

  # check dimension criterion for information criterion
  if(n <= p + 1) stop('Information criterion only works when n >  p + 1')

  if(missing(wn)) {wn = log(n)/n} # If weight is not specifided, use log(n)/n(BIC)
  resid <- lm(y~X)$residuals / sqrt( 1 - (p + 1) /n)
  SSE <- sum(resid^2) *(1 - (p + 1) / n)

  sigma2_hat <- mean(resid^2)
  mu4_hat <- mean(resid^4)
  mu6_hat <- mean(resid^6)
  mu8_hat <- mean(resid^8)

  sd = sqrt((n - p - 1)  * (mu4_hat -  sigma2_hat^2))


  #Caculate covariance for constructing Joint confidence region#
  covar <- matrix(c(mu4_hat - sigma2_hat^2, mu6_hat - sigma2_hat*mu4_hat,
                    mu6_hat - sigma2_hat*mu4_hat, mu8_hat - mu4_hat^2), 2, 2)
  grad <- matrix(c(1,-2*sigma2_hat,0,1), 2, 2)
  var_delta <- grad %*% covar %*% t(grad) / (n - p -1)
  invcov <- solve(var_delta)

  # Function to optimize ##
  fn <- function(x, a, b){
    x1 <- x[1]
    x2 <- x[2]
    if(is.null(a)){
      1- pnorm(b/(n-p-1), mean = x[1], sd = sqrt(x[2]/(n-p-1)))
    } else if(is.null(b)){
      pnorm(a/(n-p-1), mean = x[1], sd = sqrt(x[2]/(n-p-1)))
    } else {
      pnorm(a/(n-p-1), mean = x[1], sd = sqrt(x[2]/(n-p-1))) -
        pnorm(b/(n-p-1), mean = x[1], sd = sqrt(x[2]/(n-p-1)))
    }
  }


  nfn <- function(x, a, b){
    x1 <- x[1]
    x2 <- x[2]
    if(is.null(a)){
      pnorm(b/(n-p-1), mean = x[1], sd = sqrt(x[2]/(n-p-1))) - 1
    } else if(is.null(b)){
      -pnorm(a/(n-p-1), mean = x[1], sd = sqrt(x[2]/(n-p-1)))
    } else {
      -pnorm(a/(n-p-1), mean = x[1], sd = sqrt(x[2]/(n-p-1))) +
        pnorm(b/(n-p-1), mean = x[1], sd = sqrt(x[2]/(n-p-1)))
    }
  }

  # Constraints on parameter optimatin ##
  ineq <- function(x){
    z1 <-qchisq(1-alpha, 2)- t(x-x0) %*% invcov %*% (x-x0) # confidence region
    return(z1)
  }

  x0 <- c(sigma2_hat, mu4_hat-sigma2_hat^2) # center of confidence region

  ci_cal_joint <- function(a, b){
    res1 <- auglag(x0, fn, lower = c(1e-10,1e-10), hin = ineq, a = a, b = b)
    res2 <- auglag(x0, nfn, lower = c(1e-10,1e-10), hin = ineq, a = a, b= b)
    return(c(res1$value, -res2$value))
  }


  model <- glmpath(X, y, family = gaussian)
  df = model$df
  Dlambda <- model$deviance - SSE

  m <- length(model$lambda)
  prob <- rep(0, m)
  ci_mat <- matrix(nrow = 2, ncol = m)
  colnames(ci_mat) <- model$lambda
  names(prob) <- model$lambda
  for(k in 1:m){
    A <- which(df < df[k])
    B <- which(df > df[k])
    C <- which(df == df[k])

    I_k <- Dlambda[k] == min(Dlambda[C]) * 1
    ## Start with A is empty set
    if(length(A) == 0){
      a = NULL
      b <- max( (Dlambda[B] * exp( wn * (df[B] - df[k]) ) - Dlambda[k]) /
                  (1 - exp(wn *(df[B] - df[k]))))
      prob_k <- min(1 - pnorm(b, mean = SSE, sd = sd), I_k)
      ## determine the CI
      if( b > 0 & I_k == 1 ){
        ci_k <- ci_cal_joint(a, b)
      } else if(I_k == 1){
        ci_k = c(1, 1)
      } else {
        ci_k = c(0, 0)
      }

    }
    ## Start B is an empty set
    else if(length(B) == 0){
      b = NULL
      a <- min( (Dlambda[A] * exp( wn * (df[A] - df[k]) ) - Dlambda[k]) /
                  (1 - exp( wn *(df[A] - df[k])) )  )
      prob_k <- min(pnorm(a, mean = SSE, sd = sd), I_k)
      # determine the CI
      if( a > 0 & I_k == 1 ){
        ci_k <- ci_cal_joint(a, b)
      }  else {
        ci_k = c(0, 0)
      }
    }
    ## Neither A nor B are empty set
    else{
      a <- min( (Dlambda[A] * exp( wn * (df[A] - df[k]) ) - Dlambda[k]) /
                  (1 - exp( wn *(df[A] - df[k])) )  )
      b <- max( (Dlambda[B] * exp( wn * (df[B] - df[k]) ) - Dlambda[k]) /
                  (1 - exp(wn *(df[B] - df[k]))))
      prob_k <- pnorm(a, mean = SSE, sd = sd) - pnorm(b, mean = SSE, sd = sd)
      prob_k <- ifelse(prob_k >= 0, prob_k, 0)
      prob_k <- min(prob_k, I_k)
      if(a > b & I_k == 1) {
        ci_k <- ci_cal_joint(a, b)
      } else{
        ci_k = c(0, 0)
      }
    }
    # print(c(a, b))
    prob[k] <- prob_k
    ci_mat[,k] <- ci_k
  }

  res <- list(prob = prob, CI = ci_mat, model = model)
  res$alpha = alpha
  res$wn = wn
  return(res)
}
