#' Conditional Distribution based on normal assumption
#' @description Estimate conditional distribution of optimal
#' tuning paramter based on normality assumption
#' @importFrom glmpath glmpath
#' @importFrom stats optimize
#' @param X    input matrix or design matrix
#' @param y    response variable
#' @param wn   weight function, default is \eqn{log(n) / n}, which correponds to
#' BIC criterion. For AIC, choose \eqn{2 / n}.
#' @param alpha value to specify confidence level.  it
#' corrsponds confidence interval \eqn{ (1 - alpha) * 100\% }
#' @param ... Other arguments that can be passed to \code{glmpath}
#' @return
#' \item{prob}{Estimated conditional probablity of tuning parameter being
#' selected as the optimal tuning paramter}
#' \item{CI}{Confidence intervals for conditional probality based on normal
#' assumption}
#' \item{model}{a \code{glmpath} object, which is the same the fit
#' \code{glmpath(X, y, family = 'gaussian')}}
#' \item{alpha}{the specified alpha level}
#' \item{wn}{the specified wn}
#' @export
#' @examples
#' set.seed(0622)
#' X <- matrix(rnorm(50*10), 50, 10)
#' y <- X[,1:3] %*% c(1, 0.5, 1) + rnorm(50)
#' chisq_ci(X,y)



chisq_ci <- function(X, y, wn, alpha = 0.1, ...){

  # Check dimensions #
  if(is.null(dim(X))) stop("X should be a matrix");
  dimy=dim(y)
  nrowy=ifelse(is.null(dimy),length(y),dimy[1])
  if(nrowy!=dim(X)[1])
    stop(paste('number of observations does not match for X and y'))


  p <- ncol(X)
  n <- nrow(X)
  if(missing(wn)) {wn = log(n)/n} # If weight is not specifided, use log(n)/n(BIC)

   # check dimension criterion for information criterion
   if(n <= p + 1) stop('Information criterion only works when n >  p + 1')

  ci_cal_norm <- function(a, b, SSE, alpha){
    # Function to cacluate CI
    uqc <- qchisq(1 - alpha / 2, df = n-p-1)
    lqc <- qchisq(alpha / 2, df = n-p-1)

    lb <- pchisq( a / (SSE / uqc), df = n - p - 1) -
      pchisq(b / (SSE / uqc) , df = n - p - 1)

    ub <- pchisq( a / (SSE / lqc), df = n - p - 1) -
      pchisq(b / (SSE / lqc) , df = n - p - 1)
    ci <- c(min(lb, ub), max(lb, ub))

    gt <- function(t){
      pchisq(a / t, df = n - p - 1) - pchisq(b / t, df = n - p - 1)
    }

    maxim <- optimize(gt, maximum = T, interval = SSE / c(uqc, lqc))$objective
    ci[2] <- maxim

    return(ci)
  }


  SSE <- sum(lm(y~X)$residuals^2)
  sigmahat2 <- SSE / (n - p - 1)

  model <- glmpath(X, y, family = gaussian, ...)
  df <- model$df
  Dlambda <- model$deviance - SSE

  m <- length(model$lambda)
  prob <- rep(0, m)
  names(prob) <- model$lambda

  ci_mat <- matrix(0, nrow = 2, ncol = m)
  colnames(ci_mat) <- model$lambda

  for(k in 1:m){
    A <- which(df < df[k])
    B <- which(df > df[k])
    C <- which(df == df[k])

    I_k <- Dlambda[k] == min(Dlambda[C]) * 1
    if(length(A) == 0){
      b <- max( (Dlambda[B] * exp( wn * (df[B] - df[k]) ) - Dlambda[k]) /
                  (1 - exp(wn *(df[B] - df[k]))))
      prob_k <- min(1 - pchisq(b / sigmahat2 , df = n - p - 1), I_k)
      if( b > 0 & I_k == 1){
        ci <- ci_cal_norm(a = 100000 * b, b = b,SSE, alpha)
      } else if (I_k == 1){
        ci = c(1, 1)
      } else {
 	ci = c(0, 0)
      }
    }

   else if(length(B) == 0){
      a <- min( (Dlambda[A] * exp( wn * (df[A] - df[k]) ) - Dlambda[k]) /
                  (1 - exp( wn *(df[A] - df[k])) )  )
      prob_k <- min(pchisq(a / sigmahat2 , df = n - p - 1), I_k)
      if( a > 0 & I_k == 1){
        ci <- ci_cal_norm(a = a, b = 0,SSE, alpha)
      } else {
        ci = c(0, 0)
      }
    } else{
      a <- min( (Dlambda[A] * exp( wn * (df[A] - df[k]) ) - Dlambda[k]) /
                  (1 - exp( wn *(df[A] - df[k])) )  )
      b <- max( (Dlambda[B] * exp( wn * (df[B] - df[k]) ) - Dlambda[k]) /
                  (1 - exp(wn *(df[B] - df[k]))))
      if(a > b & I_k == 1) {
        prob_k <- pchisq(a / sigmahat2 , df = n - p - 1) -
          pchisq(b / sigmahat2 , df = n - p - 1)
        ci <- ci_cal_norm(a, b, SSE, alpha)
      }  else {
        prob_k <- 0
        ci <- c(0, 0)
      }
      prob_k <- ifelse(prob_k >= 0, prob_k, 0)
      prob_k <- min(prob_k, I_k)
    }

    prob[k] <- prob_k
    ci_mat[, k] <- ci
  }

  res <- list(prob = prob, CI = ci_mat, model = model)
  res$alpha = alpha
  res$wn = wn
  return(res)
}
