#' Conditional Distribution of optimal tuning paramter
#' @description Estimate conditional distribution of optimal
#' tuning paramter based on either asymptotic distribution or normal assumption
#' @param X    input matrix or design matrix
#' @param y    response variable
#' @param wn   weight function, default is \eqn{log(n) / n}, which correponds to
#' BIC criterion. For AIC, choose \eqn{2 / n}.
#' @param alpha value to specify confidence level.  it
#' corrsponds confidence interval \eqn{ (1 - alpha) * 100\% }
#' @param method used to estimate conditional distribution and confidence interval
#' @param resample whether a bootstrap sample is drawn to produce solution
#' path
#' @param ... Other arguments that can be passed to \code{glmpath}
#' @return
#' \item{prob}{Estimated conditional probablity of tuning parameter being
#' selected as the optimal tuning paramter}
#' \item{CI}{Confidence intervals for conditional probality based on normal
#' assumption}
#' \item{model}{a \code{glmpath} object, which is the same the fit
#' \code{glmpath(X, y, family = 'gaussian')}}
#' \item{alpha}{the specified alpha level}
#' \item{method}{the specified method}
#' \item{wn}{the specified wn}
#' @export
#' @examples
#' set.seed(0622)
#' X <- matrix(rnorm(50*10), 50, 10)
#' y <- X[,1:3] %*% c(1, 0.5, 1) + rnorm(50)
#' ivs_model = ivs(X,y, method = "norm")
#' summary(ivs_model)
#' plot(ivs_model)
#' 
#' # If a single model is favored by conditional probality,
#' # Using resampling to check solution path's unceintanty
#' ivs_model = ivs(X,y, method = "norm", resample = TRUE)
#' summary(ivs_model)
#' plot(ivs_model, label = TRUE)


ivs <- function(X, y, wn, alpha = 0.1, method = c('normal','asymptotic'),
                resample = FALSE, ...){
  
  method = match.arg(method)
  
  n = nrow(X)
  ny = length(y)
  if(n != ny){
    stop("number of observations in y does 
               not equal to the number of rows of X")
  }
  
  if(resample == T){
    index = sample(1:n, rep = T)
    X = X[index, ]
    y = y[index]
  }
  
  if(method == 'normal'){
    res <- chisq_ci(X, y, wn, alpha, ...)
  } else if(method == 'asymptotic'){
    res <- asym_ci(X, y, wn, alpha, ...)
  }

  res$method <- method
  class(res) <- 'ivs'
  return(res)
}

