#' Plot the solution path with the conditional distribution overlaid
#' @description Estimate conditional distribution of optimal
#' tuning paramter based on asymptotic distribution
#' @importFrom graphics matplot
#' @param x    A \code{ivs} object produced by \code{ivs(X, y)}
#' @param xvar   name of X-axis. "norm"
#' plots against the L1-norm of the coefficients,
#' "lambda" against the log-lambda sequence
#' @param label If \code{TRUE}, label the solution path with variable sequences
#' numbers
#' @param abovex If \code{True}, the conditional distribution will be plotted
#' above X-axis. Otherwise, below X-axis.
#' @param emphasize If \code{True}, model with an upper confidence
#' interval greater than threshold will be emphasized on the plot.
#' @param thresh threshold for upper confidence interval
#' @param ... Other arguments that can be passed to \code{matplot}
#' @export
#' @examples
#' set.seed(0622)
#' X <- matrix(rnorm(50*10), 50, 10)
#' y <- X[,1:3] %*% c(1, 0.5, 1) + rnorm(50)
#' res <- ivs(X,y, method = "norm")
#' plot(res, label = TRUE)


plot.ivs <- function(x, xvar = c("l1norm", 'lambda'), label = F,
                     abovex = TRUE, emphasize = T, thresh = 0.5, ...) {

  model <- x$model
  xvar = match.arg(xvar)
  beta = model$b.predictor[,-1]
  if(xvar == 'lambda'){
    matplot(log(model$lambda), beta, xlab = 'log(lambda)', type = 'l',
            ylab = 'Coefficients', lty = 1, ...)
    if(label == T) { # add label
      which = which(beta[nrow(beta), ] != 0)
      nnz=length(which)
      xpos=min(log(model$lambda[1:(length(model$lambda) -1)]))
      pos=2
      xpos=rep(xpos,nnz)
      ypos=beta[nrow(beta), ]
      text(xpos,ypos,paste(which),cex=.5,pos=pos)
    }

    # Plot the conditional distribution#
    names(x$prob) = log(as.numeric(names((x$prob))))
    if(abovex == T){
      add.hist(max(beta), x$prob)
    } else {add.hist(min(beta), x$prob)}

  }

  if(xvar == 'l1norm'){
    l1norm <- rowSums(abs(beta))
    matplot(l1norm, beta, xlab = '|beta|', type = 'l',
            ylab = 'Coefficients', lty = 1)
    if(label == T) { # add label
      which = which(beta[nrow(beta), ] != 0)
      nnz=length(which)
      xpos=max(l1norm)
      pos=4
      xpos=rep(xpos,nnz)
      ypos=beta[nrow(beta), ]
      text(xpos,ypos,paste(which),cex=.5,pos=pos)
    }

    # Plot the conditional distribution#
    names(x$prob) = l1norm
    if(abovex == T){
      add.hist(max(beta), x$prob)
    } else {add.hist(min(beta), x$prob)}
  }

   if(emphasize == T){
     emph(x, thresh)
   }
}

add.hist <- function(ymax, tab){
  reps <- sum(tab)
  axis(4, at=seq(0, ymax, length.out = 11),labels=round(seq(0, 1, 0.1),digits=2),
       col.axis="blue", las=1, cex.axis=0.5, tck=-.01)
  # add a title for the right axis
  mtext("Percentage", side=4, line=1.1, cex.lab=1,las=0, padj = 2, col ="blue")
  xvar <- as.numeric(names(tab))
  for(k in 1:length(tab))
  {
    lines(c(xvar[k], xvar[k]), c(0, tab[k]/reps * ymax), col = 1)
  }
}

emph <- function(x, thresh){
  loc <- which(x$CI[2, ] > thresh)
  if(length(loc) > 0){
  xvar <- as.numeric(names(x$prob))
  labels <- seq(1, length(loc))
  axis(3, at=xvar[loc], tick = T, labels = labels,
       col.axis="red", las=1, cex.axis=0.5, tck=-.01)
   }
}

