#' Summary of estimated conditional distribution
#' @description Summary of estimated conditional distribution and
#' confidence interval
#' @param object  A \code{ivs} object produced by \code{ivs(X, y)}
#' @param tau threshold for probablity, summary only output probability greater
#' than \code{tau}
#' @param digits minimal number of significant digits to print
#' @param ... other arguments
#' @export
#' @details
#' The function returns a summary table based on threshold tau. The first
#' and second column corresponds the tuning paramter and estimated probability
#' respectively. And the last two columns are correponding confidence interval
#' @examples
#' set.seed(0622)
#' X <- matrix(rnorm(50*10), 50, 10)
#' y <- X[,1:3] %*% c(1, 0.5, 1) + rnorm(50)
#' res <- ivs(X,y, method = "norm")
#' summary(res)


summary.ivs <- function(object, tau = 0.001, digits = 3, ...){
  alpha <- object$alpha
  M <- cbind(as.numeric(names(object$prob)), object$prob,
             object$CI[1, ],
             object$CI[2, ])
  inds <- which(object$prob >= tau)
  if(length(inds) == 0) stop('No estimated probablity is greater than specified
                             tau')

  M <- matrix(M[inds, ], ncol = 4)

  rownames(M) <- NULL
  colnames(M) <- c('Lambda', 'Prob', paste(alpha / 2 * 100, "%", sep = ""),
                   paste((1 - alpha / 2)*100, "%", sep ="") )
  print(M, digits = digits)
}


