#' Conditional Distribution based on bootstrap method
#' @description Estimate conditional distribution of optimal
#' tuning paramter based on bootstrap method
#' @importFrom glmpath glmpath
#' @param X    input matrix or design matrix
#' @param y    response variable
#' @param wn   weight function, default is \eqn{log(n) / n}, which correponds to
#' BIC criterion. For AIC, choose \eqn{2 / n}.
#' @param reps Number of bootstrap sample, defaut is \code{5000}
#' @param ... Other arguments that can be passed to \code{glmpath}
#' @return
#' \item{prob}{Estimated conditional probablity of tuning parameter being
#' selected as the optimal tuning paramter}
#' \item{model}{a \code{glmpath} object, which is the same the fit
#' \code{glmpath(X, y, family = 'gaussian')}}
#' @export
#' @examples
#' set.seed(0622)
#' X <- matrix(rnorm(50*10), 50, 10)
#' y <- X[,1:3] %*% c(1, 0.5, 1) + rnorm(50)
#' boot_lambda(X,y)



boot_lambda <- function(X, y, wn, reps = 5000, ...){


  # Check dimensions #
  if(is.null(dim(X))) stop("X should be a matrix");
  dimy=dim(y)
  nrowy=ifelse(is.null(dimy),length(y),dimy[1])
  if(nrowy!=dim(X)[1])
    stop(paste('number of observations does not match for X and y'))

  p <- ncol(X)
  n <- nrow(X)
  if(missing(wn)) {wn = log(n)/n} # If weight is not specifided, use log(n)/n(BIC)

  # Nonparametric Bootstrap to approximate the distribution of $\lambda$
  # conditional on \beta_ols #


 
  model <- glmpath(X, y, family = gaussian, ...)# fit lasso using (y*, x)
  MSE <- model$deviance / n

  slm <- lm(y ~ X)
  beta.ols <- slm$coef
  e <- slm$residuals / sqrt(1 - (p + 1) /n)
  ypred <- predict(slm)

  lambda.boot <- rep(0, reps)
  for(i in 1:reps){
    e2 = sample(e, n, replace = T)
    yb <- ypred + qr.resid(slm$qr, e2)
    
    MSE2 = MSE + (sum(yb^2)  - sum(y^2)) / n
    value = log(MSE2) + wn * model$df
    lambda.boot[i] <-model$lambda[which.min(value)]
  }
  return(list(prob = t(as.matrix(table(lambda.boot) / reps)), model = model))
}
